#!/usr/bin/env python3
# encoding: utf-8
"""
ADF2.py

Created by Jaime Stark on 2011-11-10.
Copyright (c) 2011 Jaime Stark. All rights reserved.

Python Note: Currently, file output is not compatible with Python 2.7.
"""

import sys, os, math, getopt, sqlite3
from math import sqrt


help_message = '''
AutoDockFilter v2.0  Copyright (C) 2011  Jaime Stark
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions; see `LICENSE.txt' for details.

Article Reference:

J. Stark and R. Powers* (2008) "Rapid Prediction of Protein-Ligand
  Co-structures by Combining NMR Chemical Shift Perturbations with
  AutoDock", J. Amer. Chem. Soc., 130(2): 535-545.

Usage: ADF2.py [OPTIONS]

-D "path"	Path to directory for output                          [.]
-d "file"   Input AutoDock docking log filename         [docking.dlg]
-p "file"   Input protein PDB structure filename        [protein.pdb]
-c "file"   Chemical shift perturbations filename           [CSP.txt]
-o "file"   Results table output filename               [results.txt]
-x "file"   Results xml output filename                 [results.xml]
-n "int"    Number of ADF costructures to generate                [1]
-u "float"  Standard deviations from the mean for CSP upper bound [2]
-l "float"  Standard deviations from the mean for CSP lower bound [2]
-y "float"  Pseudodistance for CSP upper bound                [3.0 A]
-z "float"  Pseudodistance for CSP lower bound                [6.0 A]
-k "float"  Force constant for NMR Violation Energy            [25.0]
-r "float"  RMSD cutoff for a ligand to belong to a cluster   [2.0 A]
-a          Turns on the generation of best AutoDock costructure   
            based on binding energy
-h          Prints this help message'''


## Import chemical shift perturbations for consensus binding site
def csp_file(filename):
	print("Importing CSPs")
	cur.execute('CREATE TABLE IF NOT EXISTS csp \
		    	(id INTEGER PRIMARY KEY AUTOINCREMENT, ResName TEXT, ResNum INT,\
		 		 CSPtype INT, CSPMag REAL)')
	try:
		file = open(filename)
		for read_line in file:
			if not read_line.strip():
				continue
			else:
				line = read_line.strip()
				csplist = (line.rsplit(" "))
				cur.execute("INSERT INTO csp VALUES(NULL,?,?,?,?)",csplist)
		file.close()
	except IOError as e:
		print(filename, "does not exist.")
		sys.exit()
	return


## Import AutoDock docking log
def ad_file(filename):
	print("Importing AutoDock docking log")
	global runs, ligandatomnum
	modelnum = 0
	cur.execute("CREATE TABLE IF NOT EXISTS inputlig \
				(id INTEGER PRIMARY KEY AUTOINCREMENT, AtomID TEXT, LigandID TEXT, ChainID TEXT, \
				ResNum INT, AtomNum INT, XCoord REAL, YCoord REAL, ZCoord REAL)")
	cur.execute("CREATE TABLE IF NOT EXISTS dockedlig \
				(id INTEGER PRIMARY KEY AUTOINCREMENT, ModelNum INT, BindingEnergy REAL, AtomNum INT, \
				XCoord REAL, YCoord REAL, ZCoord REAL)")
	cur.execute("CREATE TABLE IF NOT EXISTS rmsdtable \
				(id INTEGER PRIMARY KEY AUTOINCREMENT, Rank INT, SubRank INT, Run INT, \
				BindingEnergy REAL, ClusterRMSD REAL, RefRMSD REAL)")
	try:
		file = open(filename)
		for read_line in file:
			## Read coordinate values for the input ligand
			if (read_line.find("INPUT-LIGAND-PDBQT: ATOM") != -1) or (read_line.find("INPUT-LIGAND-PDBQT: HETATM") != 0-1):
				anum = int(read_line[27:31])
				atID = read_line[31:36]
				ligID = read_line[37:40]
				chID = read_line[42:43]
				rnum = int(read_line[43:47])
				xcoor = float(read_line[51:59])
				ycoor = float(read_line[59:67])
				zcoor = float(read_line[67:75])
				inputlist = atID,ligID,chID,rnum,anum,xcoor,ycoor,zcoor
				ligandatomnum = anum
				cur.execute("INSERT INTO inputlig VALUES(NULL,?,?,?,?,?,?,?,?)", inputlist)
		
			## Determine the total number of runs (i.e. number of docked models)
			## Note: Currently only reads number of runs if using hybrid GA-LS
			elif read_line.find('DPF> ga_run') != -1:
				runs = int(read_line[12:16])
		
			## Identifies when a docked model is encountered
			elif read_line.find("DOCKED: MODEL") != -1:
				modelnum = int(read_line[16:23])	
		
			## Reads the final binding energy for the docked model
			elif (read_line.find("DOCKED: USER    Estimated Free Energy of Binding") \
			  	!= -1) & (modelnum > 0):
				bindingenergy = float(read_line[54:61])		
		
			## Reads the coordinate values for each docked model
			elif (read_line.find("DOCKED: ATOM") != -1) & (modelnum > 0):
				anum = int(read_line[14:19])
				xcoor = float(read_line[38:46])
				ycoor = float(read_line[46:54])
				zcoor = float(read_line[54:62])
				dockedlist = modelnum,bindingenergy,anum,xcoor,ycoor,zcoor
				cur.execute("INSERT INTO dockedlig VALUES(NULL,?,?,?,?,?,?)", dockedlist)
			
			## Stops reading data for the docked model
			elif read_line.find("DOCKED: ENDMDL") != -1:
				modelnum = 0
				
			## Reads RMSD Table
			elif (read_line.find("RANKING") != -1):
				line = read_line.strip()
				rmsdtab = line.split()
				rmsdtab.pop()
				cur.execute("INSERT INTO rmsdtable VALUES(NULL,?,?,?,?,?,?)", rmsdtab)
		file.close()
	except IOError as e:
		print(filename, "does not exist.")
		sys.exit()
	return	


## Import protein pdb coordinates
def pdb_file(filename):
	print("Importing PDB coordinates of protein")
	cur.execute("CREATE TABLE IF NOT EXISTS pdb \
				(id INTEGER PRIMARY KEY AUTOINCREMENT, AtomNum INT, AtomName TEXT, AltLoc TEXT,\
				ResName TEXT, ChainID TEXT, ResNum INT, ResIns TEXT,\
				XCoord REAL, YCoord REAL, ZCoord REAL, Occupancy REAL, TFactor REAL)")
	try:
		file = open(filename)
		for read_line in file:			
			## Read in PDB atom coordinates to PDB file
			if (read_line[0:6] == 'ATOM  '):
				atomnum = int(read_line[6:11])									# Atom serial number
				atomname = read_line[12:16]										# Atom name
				altloc = read_line[16:17]										# Alternate location indicator
				resname = read_line[17:20]										# Residue name
				chainID = read_line[21:22]										# Chain identifier
				resnum = int(read_line[22:26])									# Residue sequence number
				resins = read_line[26:27]										# Code for insertion of residues
				xcoord = float(read_line[30:38])								# X Coordinates
				ycoord = float(read_line[38:46])								# Y Coordinates
				zcoord = float(read_line[46:54])								# Z Coordinates
				occupancy = float(read_line[54:60])								# Occupancy
				tfactor = float(read_line[60:66])								# Temperature factor
				pdblist = atomnum,atomname,altloc,resname,chainID,\
					  	resnum,resins,xcoord,ycoord,zcoord,occupancy,tfactor
				cur.execute("INSERT INTO pdb VALUES(NULL,?,?,?,?,?,?,?,?,?,?,?,?)", pdblist)
		file.close()
	except IOError as e:
		print(filename, "does not exist.")
		sys.exit()
	return


## Calculate the mean and standard deviation in 1D list
def meanstdev(alist):
	n, mean, stdev = len(alist), 0, 0	
	for a in alist:
		mean = mean + a
	mean = mean / float(n)																		# Calculate mean			
	for a in alist:
		stdev = stdev + ((a - mean)**2)
	if n == 1:
		stdev = 0.0
	elif n > 1:
		stdev = sqrt(stdev / float(n-1))														# Calculate standard deviation
	return mean, stdev


## Calculate Mean and Standard Deviation of CSPs
def CSPmeanstdev():
	print("Calculating mean and standard deviation of CSPs")
	global cspmean, cspstdev, cspupper, csplower, csprange
	cspmaglist = []
	cur.execute("SELECT * FROM csp")
	rows = cur.fetchall()
	for row in rows:
		cspmaglist.append(float(row[4]))
	cspmeanstdev = meanstdev(cspmaglist)
	cspmean = cspmeanstdev[0]										# Mean of CSPs
	cspstdev = cspmeanstdev[1]										# Standard deviation of CSPs
	cspupper = cspmean + (csp_stdev_upper*cspstdev)					# Upper threshold (csp_stdev_upper is standard deviations)
	csplower = cspmean - (csp_stdev_lower*cspstdev)					# Lower threshold (csp_stdev_lower is standard deviations)
	csprange = csplower - cspupper									# Range between thresholds
	return


## Convert CSPs to pseudodistances
def PseudoDistance():
	print("Converting CSPs to pseudodistances")
	cspdist = []
	cur.execute("CREATE TABLE IF NOT EXISTS pseudodist\
				  (id INTEGER PRIMARY KEY AUTOINCREMENT, ResName TEXT, ResNum INT, CSPtype INT, CSPDist REAL)")
	cur.execute("SELECT * FROM csp")
	rows = cur.fetchall()
	for row in rows:
		# If only one CSP or CSPs are identical, set pseudodistance to lower bounds (Default: 3 angstroms)
		if cspstdev == 0.0:
			cspdist = row[1], row[2], row[3], pseudo_lower
			cur.execute("INSERT INTO pseudodist VALUES(NULL,?,?,?,?)", cspdist)
		# Set all CSPs above the upper threshold to a pseudodistance to lower bounds (Default: 3 angstroms)
		elif float(row[4]) > cspupper:
			cspdist = row[1], row[2], row[3], pseudo_lower
			cur.execute("INSERT INTO pseudodist VALUES(NULL,?,?,?,?)", cspdist)
		## Set remaining CSPs to linear normalized CSP based on upper and lower bounds (Default: 6 angstroms to 3 angstroms)
		else:
			pseudodist = (float(row[4])-(cspupper+abs(csprange)))/(csprange/float(pseudo_upper-pseudo_lower))
			cspdist = row[1], row[2], row[3], pseudodist
			cur.execute("INSERT INTO pseudodist VALUES(NULL,?,?,?,?)", cspdist)
	return	


## Calculate shortest distance of each perturbed residue to each docked ligand and compares to pseudodistances
def ShortestDistance():
	print("Comparing docked ligands to pseudodistances")
	cur.execute("CREATE TABLE IF NOT EXISTS hshortdist \
				(id INTEGER PRIMARY KEY AUTOINCREMENT, ResName TEXT, ResNum INT, AtomType TEXT,\
				 ModelNum INT, BindingEnergy REAL, ShortestDist REAL, PseudoDist REAL, DeltaDist REAL)")
	cur.execute("CREATE TABLE IF NOT EXISTS nshortdist \
				(id INTEGER PRIMARY KEY AUTOINCREMENT, ResName TEXT, ResNum INT, AtomType TEXT,\
				 ModelNum INT, BindingEnergy REAL, ShortestDist REAL, PseudoDist REAL, DeltaDist REAL)")
	cur.execute("CREATE TABLE IF NOT EXISTS shortdist \
				(id INTEGER PRIMARY KEY AUTOINCREMENT, ResName TEXT, ResNum INT, AtomType TEXT,\
				 ModelNum INT, BindingEnergy REAL, ShortestDist REAL, PseudoDist REAL, DeltaDist REAL)")		
	cur.execute("SELECT * FROM pseudodist")
	crows = cur.fetchall()
	for crow in crows:
		resname, resnum, pseudodist = crow[1], crow[2], crow[4]
		cur.execute("SELECT * FROM pdb WHERE (ResName=? AND ResNum=? AND AtomName = ' H  ')", (resname,resnum))
		hprows = cur.fetchall()
		for hprow in hprows:
			hpxcoord, hpycoord, hpzcoord = hprow[8], hprow[9], hprow[10]
			run = 1
			while run <= runs:
				cur.execute("SELECT * FROM dockedlig WHERE ModelNum=? ", (run,))
				drows = cur.fetchall()
				distcalc = []
				for drow in drows:
					distcalc.append(sqrt(((hpxcoord - drow[4])**2) + \
							   ((hpycoord - drow[5])**2) + \
							   ((hpzcoord - drow[6])**2)))					
				modelnum = drow[1]
				bindingenergy = drow[2]
				distcalc.sort()
				deltadist = distcalc[0] - pseudodist
				hprotlist = resname, resnum, "H", modelnum, bindingenergy, distcalc[0], pseudodist, deltadist
				cur.execute("INSERT INTO hshortdist VALUES(NULL,?,?,?,?,?,?,?,?)", hprotlist)
				run += 1
	
		cur.execute("SELECT * FROM pdb WHERE (ResName=? AND ResNum=? AND AtomName = ' N  ')", (resname,resnum))
		nprows = cur.fetchall()
		for nprow in nprows:
			npxcoord, npycoord, npzcoord = nprow[8], nprow[9], nprow[10]
			run = 1
			while run <= runs:
				cur.execute("SELECT * FROM dockedlig WHERE ModelNum=? ",(run,))
				drows = cur.fetchall()
				distcalc = []
				for drow in drows:
					distcalc.append(sqrt(((npxcoord - drow[4])**2) + \
							   ((npycoord - drow[5])**2) + \
							   ((npzcoord - drow[6])**2)))
				modelnum = drow[1]
				bindingenergy = drow[2]
				distcalc.sort()
				deltadist = distcalc[0] - pseudodist
				nprotlist = resname, resnum, "N", modelnum, bindingenergy, distcalc[0], pseudodist, deltadist
				cur.execute("INSERT INTO nshortdist VALUES(NULL,?,?,?,?,?,?,?,?)", nprotlist)
				run += 1
	
	cur.execute("SELECT * FROM hshortdist")
	hdist = cur.fetchall()
	for h in hdist:
		resname, resnum, modelnum, bindingenergy, hdistcalc, hpseudo, hdelta = h[1],h[2],h[4],h[5],h[6],h[7],h[8]
		cur.execute("SELECT * FROM nshortdist WHERE (ResName=? AND ResNum=? AND ModelNum=?)", (resname,resnum,modelnum))
		ndist = cur.fetchone()
		ndistcalc, npseudo, ndelta = ndist[6], ndist[7], ndist[8]
		if hdistcalc <= ndistcalc:
			shortdist = resname, resnum, "H", modelnum, bindingenergy, hdistcalc, hpseudo, hdelta
			cur.execute("INSERT INTO shortdist VALUES(NULL,?,?,?,?,?,?,?,?)", shortdist)
		elif ndistcalc < hdistcalc:
			shortdist = resname, resnum, "N", modelnum, bindingenergy, ndistcalc, npseudo, ndelta
			cur.execute("INSERT INTO shortdist VALUES(NULL,?,?,?,?,?,?,?,?)", shortdist)
	return


## Calculate NMR violation energy
def NMREnergy():
	print("Calculating NMR violation energy")
	run = 1
	cur.execute("CREATE TABLE IF NOT EXISTS nmrenergy \
				(id INTEGER PRIMARY KEY AUTOINCREMENT, ModelNum INT, BindingEnergy REAL, NMREnergy REAL)")
	while run <= runs:
		cur.execute("SELECT * FROM shortdist WHERE ModelNum=?", (run,))
		rows = cur.fetchall()
		dist2_sum = 0
		for row in rows:
			if row[8] <= 0:
				dist = 0
			elif row[8] > 0:
				dist = row[8]
			benergy = row[5]
			dist2 = dist**2
			dist2_sum = dist2_sum + dist2
		## The final sum of distance differences is multiplied by an NMR force constanct (Default: 25)
		venergy = dist2_sum * nmr_force_constant
		cur.execute("INSERT INTO nmrenergy VALUES(NULL,?,?,?)", (run,benergy,venergy))
		run += 1
	return


## Calculate rmsd between two lists of 3-tuple coordinates (Symmetry is ignored)
def rmsd(allcoord1,allcoord2):
	squared_dist = 0
	atom_count = 1
	for (atom1,atom2) in zip(allcoord1, allcoord2):
		squared_dist_atom = sum((coord1-coord2)**2 for (coord1,coord2) in zip (atom1,atom2))
		squared_dist = squared_dist_atom + squared_dist
		atom_count += 1
	return math.sqrt(squared_dist / float(atom_count))


## Cluster the results based on NMR energy and rmsd
def Cluster():
	print("Clustering results based on NMR energy and rmsd")
	global best_autodock,best_adf
	cur.execute("SELECT * FROM nmrenergy ORDER BY BindingEnergy ASC")
	row = cur.fetchone()
	best_autodock = row[1]
	cur.execute("SELECT * FROM nmrenergy ORDER BY NMREnergy ASC")
	row = cur.fetchone()
	best_adf = row[1]
	cur.execute("SELECT * FROM nmrenergy ORDER BY NMREnergy ASC")
	rows = cur.fetchall()
	# Calculate rmsd to first cluster (If < 2 angstroms assign to that cluster.
	# If > 2 angstroms, compare to next cluster. If it doesn't match any cluster
	# create a new cluster.
	# Should also calculate the rmsd of each molecule from the initial ligand
	# and the first ranked ligand.
	adf_rankcount = 1
	#Find coordinates of input ligand
	cur.execute("SELECT XCoord,YCoord,ZCoord FROM inputlig")
	input_coord = cur.fetchall()
	#Find coordinates of ADF best ligand
	cur.execute("SELECT XCoord,YCoord,ZCoord FROM dockedlig WHERE ModelNum=?",(best_adf,))
	adf_coord = cur.fetchall()
	#Find coordinates of AutoDock best ligand
	cur.execute("SELECT XCoord,YCoord,ZCoord FROM dockedlig WHERE ModelNum=?",(best_autodock,))
	autodock_coord = cur.fetchall()
	cur.execute("CREATE TABLE IF NOT EXISTS clusters \
				(id INTEGER PRIMARY KEY AUTOINCREMENT, ModelNum INT, Cluster INT, ClusterRank INT, \
				 BindingEnergy REAL, NMREnergy REAL, ADFRank INT, AutoDockCluster INT, RMSDCluster REAL, \
				 RMSDBestADF REAL, RMSDBestAutoDock REAL, RMSDReference REAL)")
	firstrow = True
	for row in rows:
		adf_rank = adf_rankcount							# The NMR Energy Rank
		modelnum = row[1]									# Model Number
		binding_energy = row[2]								# AutoDock Binding Energy
		nmr_energy = row[3]									# ADF NMR Energy
		# Find the AutoDock cluster ranking for current ligand
		cur.execute("SELECT Rank FROM rmsdtable WHERE Run=?",(modelnum,))
		adcluster = cur.fetchone()
		autodockcluster = adcluster[0]
		# Find coordinates of current atom in current ligand
		cur.execute("SELECT XCoord,YCoord,ZCoord FROM dockedlig WHERE ModelNum=?",(modelnum,))
		curr_coord = cur.fetchall()
		# rmsd of ligand to input ligand
		rmsd_to_reference = rmsd(curr_coord,input_coord)	
		# rmsd of ligand to lowest ADF
		rmsd_to_best_adf = rmsd(curr_coord,adf_coord)							
		# rmsd of ligand to AutoDock best
		rmsd_to_autodock = rmsd(curr_coord,autodock_coord)
		if firstrow == True:
			cur.execute("INSERT INTO clusters VALUES (NULL,?,1,1,?,?,?,?,0.00,0.00,?,?)",(modelnum,binding_energy,nmr_energy,adf_rank,autodockcluster,\
						rmsd_to_autodock,rmsd_to_reference))
			cluster, cluster_rank = 1,1
		else:
			cur.execute("SELECT * FROM clusters WHERE ClusterRank=1")
			clusterbests = cur.fetchall()
			for clusterbest in clusterbests:
				clusternum = clusterbest[2]
				clusterbestmodel = clusterbest[1]
				cur.execute("SELECT XCoord,YCoord,ZCoord FROM dockedlig WHERE ModelNum=?",(clusterbestmodel,))
				cluster_coord = cur.fetchall()
				rmsd_to_cluster = rmsd(curr_coord,cluster_coord)
				if rmsd_to_cluster <= cluster_rmsd_cutoff:
					cur.execute("SELECT MAX(ClusterRank) FROM clusters WHERE Cluster=?",(clusternum,))
					max_rank = cur.fetchone()
					cur_rank = int(max_rank[0]) + 1
					cur.execute("INSERT INTO clusters VALUES(NULL,?,?,?,?,?,?,?,?,?,?,?)",(modelnum,clusternum,cur_rank,binding_energy,\
								nmr_energy,adf_rank,autodockcluster,rmsd_to_cluster,rmsd_to_best_adf,rmsd_to_autodock,rmsd_to_reference))
					break								
			if rmsd_to_cluster > cluster_rmsd_cutoff:
				cur.execute("SELECT MAX(Cluster) FROM clusters")
				max_clusternum = cur.fetchone()
				cur_clusternum = int(max_clusternum[0]) + 1
				cur.execute("INSERT INTO clusters VALUES(NULL,?,?,1,?,?,?,?,0.00,?,?,?)",(modelnum,cur_clusternum,binding_energy,nmr_energy,\
							adf_rank,autodockcluster,rmsd_to_best_adf,rmsd_to_autodock,rmsd_to_reference))
				rmsd_to_cluster = 0.0
		firstrow = False
		adf_rankcount += 1
	return	


## Writes results table to file
def Results(filename):
	print("Generating results file")
	global max_clusters
	rfile = open(filename,'w')
	# Determine max number of adf clusters
	cur.execute("SELECT MAX(Cluster) FROM clusters")
	clustersnum = cur.fetchone()
	max_clusters = clustersnum[0]
	print("__________________________________________________________________________________________________________________________", file=rfile)
	print("| AutoDock | ADF     | ADF        | ADF    | RMSD Relative | RMSD Relative | RMSD Relative | RMSD Relative    | AutoDock |", file=rfile)
	print("| Model    | Cluster | NMR Energy | NMR    | To Best ADF   | To Best In    | To Reference  | To Best AutoDock | Binding  |", file=rfile)
	print("| Number   | Rank    | Rank       | Energy | Score         | ADF Cluster   | Ligand        | Score            | Energy   |", file=rfile)
	print("|__________|_________|____________|________|_______________|_______________|_______________|__________________|__________|", file=rfile)
	clusternum = 1
	while clusternum <= max_clusters:
		print("\n", file=rfile)
		print("ADF CLUSTER",clusternum, file=rfile)
		print("__________________________________________________________________________________________________________________________", file=rfile)
		cur.execute("SELECT * FROM clusters WHERE Cluster=? ORDER BY ClusterRank ASC ",(clusternum,))
		cluster_results = cur.fetchall()
		cluster_rmsd = []
		cluster_nmrenergy = []
		reference_rmsd = []
		for result in cluster_results:
			print("   %-4i       %-3i         %-3i      %-8.2f     %5.2f           %5.2f           %5.2f            %5.2f          %7.2f   " % \
				  (result[1],result[3],result[6],result[5],result[9],result[8],result[11],result[10],result[4]), file=rfile)
			cluster_rmsd.append(result[9])
			cluster_nmrenergy.append(result[5])
			reference_rmsd.append(result[11])
		print("__________________________________________________________________________________________________________________________", file=rfile)
		avg_cluster_rmsd = meanstdev(cluster_rmsd)
		print("                                                Average Cluster RMSD Relative to Best ADF Score  =      %5.2f  +/-  %5.2f" \
			  % (avg_cluster_rmsd[0],avg_cluster_rmsd[1]), file=rfile)
		avg_ref_rmsd = meanstdev(reference_rmsd)
		print("                                                     Average Cluster RMSD Relative to Reference  =      %5.2f  +/-  %5.2f" \
			  % (avg_ref_rmsd[0],avg_ref_rmsd[1]), file=rfile)
		avg_cluster_nmrenergy = meanstdev(cluster_nmrenergy)
		print("                                                                 Average Cluster ADF NMR Energy  =   %8.2f  +/-%7.2f" \
			  % (avg_cluster_nmrenergy[0],avg_cluster_nmrenergy[1]), file=rfile)
		clusternum += 1
	rfile.close()
	return


## Write results to xml file
def XML_Results(filename):
	print("Generating results xml file")
	global max_clusters, num_structs
	rfile = open(filename,'w')
	# Determine max number of adf clusters
	cur.execute("SELECT MAX(Cluster) FROM clusters")
	clustersnum = cur.fetchone()
	max_clusters = clustersnum[0]
	if num_adf_struct > max_clusters:
		num_structs = max_clusters
	else:
		num_structs = num_adf_struct
	clusternum = 1
	print("<adfResults>", file=rfile)
	while clusternum <= max_clusters:
		print('  <cluster id=\"%s\">' % clusternum, file=rfile)
		cur.execute("SELECT * FROM clusters WHERE Cluster=? ORDER BY ClusterRank ASC ",(clusternum,))
		cluster_results = cur.fetchall()
		cluster_rmsd = []
		cluster_nmrenergy = []
		reference_rmsd = []
		result_count = 1
		print('    <poses>', file=rfile)
		for result in cluster_results:
			if (result_count == 1) & (clusternum <= num_structs):
				print('      <pose id=\"%s\" hasCostructureFile=\"true\" costructureFile=\"ADF_Costructure%s.pdb\">' % (result_count,clusternum), file=rfile)
			else:
				print('      <pose id=\"%s\" hasCostructureFile=\"false\">' % result_count, file=rfile)
			print('        <adModelNumber>%s</adModelNumber>' % str(result[1]), file=rfile)
			print('        <adfClusterRank>%s</adfClusterRank>' % str(result[3]), file=rfile)
			print('        <adfNmrEnergyRank>%s</adfNmrEnergyRank>' % str(result[6]), file=rfile)
			print('        <adfNmrEnergy>%.2f</adfNmrEnergy>' % result[5], file=rfile)
			print('        <rmsdToBestADF>%.2f</rmsdToBestADF>' % result[9], file=rfile)
			print('        <rmsdToBestInCluster>%.2f</rmsdToBestInCluster>' % result[8], file=rfile)
			print('        <rmsdToRefLigand>%.2f</rmsdToRefLigand>' % result[11], file=rfile)
			print('        <rmsdToBestAD>%.2f</rmsdToBestAD>' % result[10], file=rfile)
			print('        <adBindingEnergy>%.2f</adBindingEnergy>' % result[4], file=rfile)
			print('      </pose>', file=rfile)
			cluster_rmsd.append(result[9])
			cluster_nmrenergy.append(result[5])
			reference_rmsd.append(result[11])
			result_count += 1
		print('    </poses>', file=rfile)
		avg_cluster_rmsd = meanstdev(cluster_rmsd)
		print('    <rmsdToBestADF>', file=rfile) 
		print('      <mean>%.2f</mean>' % avg_cluster_rmsd[0], file=rfile)
		print('      <stdev>%.2f</stdev>' % avg_cluster_rmsd[1], file=rfile)
		print('    </rmsdToBestADF>', file=rfile)
		avg_ref_rmsd = meanstdev(reference_rmsd)
		print('    <rmsdToRef>', file=rfile)
		print('      <mean>%.2f</mean>' % avg_ref_rmsd[0], file=rfile)
		print('      <stdev>%.2f</stdev>' % avg_ref_rmsd[1], file=rfile)
		print('    </rmsdToRef>', file=rfile)
		avg_cluster_nmrenergy = meanstdev(cluster_nmrenergy)
		print('    <nmrEnergy>', file=rfile)
		print('      <mean>%.2f</mean>' % avg_cluster_nmrenergy[0], file=rfile)
		print('      <stdev>%.2f</stdev>' % avg_cluster_nmrenergy[1], file=rfile)
		print('    </nmrEnergy>', file=rfile)
		clusternum += 1
	rfile.close()
	return


## Output best ADF costructures based on CSPs
def ADF_Costructures(filename1,filename2,structdir):
	print("Generating ADF costructures")
	num = 1
	while num <= num_structs:
		cur.execute("SELECT ModelNum FROM clusters WHERE (Cluster=? AND ClusterRank=1)",(num,))
		runnumber = cur.fetchone()
		modelnumber = runnumber[0]
		pfile = open(filename1)
		sfile = open(structdir + "/ADF_Costructure""%s"".pdb" % (num),'w')
		print("HEADER: AUTODOCKFILTER COSTRUCTURE",file=sfile)
		print("TITLE: PROTEIN DOCKED WITH MODEL",modelnumber, file=sfile)
		for read_line in pfile:			
			## Read in PDB atom coordinates to PDB file
			if (read_line[0:6] == 'ATOM  '):
				label = read_line[0:6]
				atomnum = read_line[6:11]									# Atom serial number
				atomname = read_line[12:16]										# Atom name
				altloc = read_line[16:17]										# Alternate location indicator
				resname = read_line[17:20]										# Residue name
				chainID = read_line[21:22]										# Chain identifier
				resnum = read_line[22:26]									# Residue sequence number
				resins = read_line[26:27]										# Code for insertion of residues
				xcoord = read_line[30:38]								# X Coordinates
				ycoord = read_line[38:46]								# Y Coordinates
				zcoord = read_line[46:54]								# Z Coordinates
				occupancy = read_line[54:60]							# Occupancy
				tfactor = read_line[60:66]								# Temperature factor
				print(label,atomnum,atomname," ",altloc,resname," ",chainID,\
						  resnum,resins,"   ",xcoord,ycoord,zcoord,occupancy,tfactor, sep="", file=sfile)
		pfile.close()
		dfile = open(filename2)
		correctligand = 0
		for read_line in dfile:
			## Identifies when the correct docked model is encountered
			if (read_line.find("DOCKED: MODEL") != -1):
				iscorrect = int(read_line[16:24])
				if modelnumber == iscorrect:
					correctligand = 1		
			## Reads the coordinate values for the correct docked model
			elif (read_line.find("DOCKED: ATOM") != -1) & (correctligand > 0):
				label = "HETATM"
				atomnum = read_line[14:19]
				atomname = read_line[20:24]
				altloc = " "
				resname = "LIG"
				chainID = "A"
				resnum = " 999"
				resins = " "
				xcoord = read_line[38:46]
				ycoord = read_line[46:54]
				zcoord = read_line[54:62]
				occupancy = "  1.00"
				tfactor = "  0.00"
				print(label,atomnum,atomname," ",altloc,resname," ",chainID,resnum,resins,\
					  "   ",xcoord,ycoord,zcoord,occupancy,tfactor, sep="", file=sfile)
			## Stops reading data for the docked model
			elif read_line.find("DOCKED: ENDMDL") != -1:
				correctligand = 0
		dfile.close()
		sfile.close()
		num += 1
	return


## Output best AutoDock costructure based on lowest binding energy
def AutoDock_Costructures(filename1,filename2,structdir):
	print("Generating best AutoDock costructure")
	pfile = open(filename1)
	sfile = open(structdir + "/AutoDock_Costructure.pdb",'w')
	print("HEADER: AUTODOCK COSTRUCTURE",file=sfile)
	print("TITLE: PROTEIN DOCKED WITH MODEL",best_autodock, file=sfile)
	for read_line in pfile:			
		## Read in PDB atom coordinates to PDB file
		if (read_line[0:6] == 'ATOM  '):
			label = read_line[0:6]
			atomnum = read_line[6:11]									# Atom serial number
			atomname = read_line[12:16]										# Atom name
			altloc = read_line[16:17]										# Alternate location indicator
			resname = read_line[17:20]										# Residue name
			chainID = read_line[21:22]										# Chain identifier
			resnum = read_line[22:26]									# Residue sequence number
			resins = read_line[26:27]										# Code for insertion of residues
			xcoord = read_line[30:38]								# X Coordinates
			ycoord = read_line[38:46]								# Y Coordinates
			zcoord = read_line[46:54]								# Z Coordinates
			occupancy = read_line[54:60]							# Occupancy
			tfactor = read_line[60:66]								# Temperature factor
			print(label,atomnum,atomname," ",altloc,resname," ",chainID,\
					  resnum,resins,"   ",xcoord,ycoord,zcoord,occupancy,tfactor, sep="", file=sfile)
	pfile.close()
	dfile = open(filename2)
	correctligand = 0
	for read_line in dfile:
		## Identifies when the correct docked model is encountered
		if (read_line.find("DOCKED: MODEL") != -1):
			iscorrect = int(read_line[16:24])
			if best_autodock == iscorrect:
				correctligand = 1		
		## Reads the coordinate values for the correct docked model
		elif (read_line.find("DOCKED: ATOM") != -1) & (correctligand > 0):
			label = "HETATM"
			atomnum = read_line[14:19]
			atomname = read_line[20:24]
			altloc = " "
			resname = "LIG"
			chainID = "A"
			resnum = " 999"
			resins = " "
			xcoord = read_line[38:46]
			ycoord = read_line[46:54]
			zcoord = read_line[54:62]
			occupancy = "  1.00"
			tfactor = "  0.00"
			print(label,atomnum,atomname," ",altloc,resname," ",chainID,resnum,resins,\
				  "   ",xcoord,ycoord,zcoord,occupancy,tfactor, sep="", file=sfile)
		## Stops reading data for the docked model
		elif read_line.find("DOCKED: ENDMDL") != -1:
			correctligand = 0
	dfile.close()
	sfile.close()
	return



def main():
	try:
		opts, args = getopt.getopt(sys.argv[1:], "D:p:d:c:o:x:n:u:l:y:z:k:r:ha")	
	except getopt.GetoptError as err:
		# print help information and exit:
		# print(err) # will print something like "option -a not recognized"
		# raise Usage(msg)
		print("Incorrect argument(s). See help file")
		sys.exit(2)
	
	## Flag argument processing
	global num_adf_struct,csp_stdev_upper,csp_stdev_lower,pseudo_lower,pseudo_upper,nmr_force_constant,\
		   autodock_struct,cluster_rmsd_cutoff
	# Assign flag variables to empty strings
	structdir,pdbfile,docklog,cspfile,resultsfile,xmlresultsfile,num_adf_struct,csp_stdev_upper,csp_stdev_lower,\
			pseudo_lower,pseudo_upper,nmr_force_constant,autodock_struct,cluster_rmsd_cutoff\
			= "","","","","","","","","","","","","",""
	for foption, fvalue in opts:
		#print(foption,fvalue)
		if foption == "-D":
			structdir = fvalue
			print("AutoDock Costructure Directory:", structdir)
		if foption == "-d":
			docklog = fvalue
			print("AutoDock Docking Log:", docklog)
		if foption == "-p":
			pdbfile = fvalue
			print("PDB File:", pdbfile)
		if foption in "-c":
			cspfile = fvalue
			print("CSP Text File:", cspfile)
		if foption in "-o":
			resultsfile = fvalue
			print("Results File:", resultsfile)
		if foption in "-x":
			xmlresultsfile = fvalue
			print("XML Results File:", xmlresultsfile)
		if foption in "-h":
			print(help_message)
			sys.exit()
		if foption in "-n":
			try:
				num_adf_struct = int(fvalue)
				print(num_adf_struct,"costructure(s) will be output")
			except ValueError:
				print("The -n flag requires a whole number integer")
				sys.exit()
		if foption in "-u":
			try:
				csp_stdev_upper = float(fvalue)
				print("The upper bounds for CSPs has been set to", csp_stdev_upper, "angstroms")
			except ValueError:
				print("The -u flag requires a number")
				sys.exit()
		if foption in "-l":
			try:
				csp_stdev_lower = float(fvalue)
				print("The lower bounds for CSPs has been set to", csp_stdev_lower, "angstroms")
			except ValueError:
				print("The -l flag requires a number")
				sys.exit()
		if foption in "-y":
			try:
				pseudo_lower = float(fvalue)
				print("The pseudodistance for the lower bound is", pseudo_lower, "angstroms")
			except ValueError:
				print("The -y flag requires a number")
				sys.exit()
		if foption in "-z":
			try:
				pseudo_upper = float(fvalue)
				print("The pseudodistance for the upper bound is", pseudo_upper, "angstoms")
			except ValueError:
				print("The -z flag requires a number")
				sys.exit()
		if foption in "-k":
			try:
				nmr_force_constant = float(fvalue)
				print("The force constant for calculating NMR Violation Energy is", nmr_force_constant)
			except ValueError:
				print("The -k flag requires a number")
				sys.exit()
		if foption in "-a":
			autodock_struct = 1
			print("The AutoDock costructure with the lowest binding energy will be output")
		if foption in "-r":
			try:
				cluster_rmsd_cutoff = float(fvalue)
				print("Docked ligands will be clustered with a", cluster_rmsd_cutoff, "angstrom cutoff")
			except ValueError:
				print("The -r flag requires a number")
				sys.exit()
	
	if structdir == "":
		structdir = "."
	if docklog == "":
		docklog = "docking.dlg"
	if cspfile == "":
		cspfile = "CSP.txt"
	if pdbfile == "":
		pdbfile = "protein.pdb"
	if resultsfile == "":
		resultsfile = "results.txt"
	if xmlresultsfile == "":
		xmlresultsfile = "results.xml"
	if num_adf_struct == "":
		num_adf_struct = 1
	if csp_stdev_upper == "":
		csp_stdev_upper = 2.0
	if csp_stdev_lower == "":
		csp_stdev_lower = 2.0
	if pseudo_lower == "":
		pseudo_lower = 3.0
	if pseudo_upper == "":
		pseudo_upper = 6.0
	if nmr_force_constant == "":
		nmr_force_constant = 25
	if autodock_struct == "":
		autodock_struct = 0
	if cluster_rmsd_cutoff == "":
		cluster_rmsd_cutoff = 2.0
		
	global cur
	con = sqlite3.connect("")
	with con:	
		cur = con.cursor()
		csp_file(cspfile)
		ad_file(docklog)
		pdb_file(pdbfile)
		CSPmeanstdev()
		PseudoDistance()
		ShortestDistance()
		NMREnergy()
		Cluster()
		Results(resultsfile)
		XML_Results(xmlresultsfile)
		if num_adf_struct > 0:
			ADF_Costructures(pdbfile,docklog,structdir)
		if autodock_struct > 0:
			AutoDock_Costructures(pdbfile,docklog,structdir)
	
	return

if __name__ == "__main__":
	sys.exit(main())
